@@ -479,6 +479,8 @@ class CobraDaemon(ThreadingTCPServer):
         self.authmod = None
         self.sflags = 0
 
+        self.allow_reuse_address = True
+
         if msgpack and json:
             raise Exception('CobraDaemon can not use both msgpack *and* json!')
 
@@ -748,8 +750,8 @@ class CobraConnectionHandler:
                 mtype,name,data = csock.recvMessage()
             except CobraClosedException:
                 break
-            except socket.error:
-                logger.warning("Cobra socket error in handleClient")
+            except socket.error as e:
+                logger.warning("Cobra socket error in handleClient. Err: %s", str(e))
                 break
 
             # If they re-auth ( app layer ) later, lets handle it...
