@@ -65,7 +65,7 @@ OPTYPE_fv = 0x18000000
 
 OPERSIZE = {
     0: (2, 4, 8),           # We will only end up here on regs embedded in opcodes
-    OPTYPE_a: (2, 4, 4),
+    OPTYPE_a: (4, 8, 8),
     OPTYPE_b: (1, 1, 1),
     OPTYPE_c: (1, 2, 2),           # 1/2   byte or word, depending on operand
     OPTYPE_d: (4, 4, 4),           # 4     double-word
@@ -255,6 +255,7 @@ cpu_AMD64 = 0x0000a000
 cpu_AESNI = 0x0000b000
 cpu_AVX   = 0x0000c000
 cpu_BMI   = 0x0000d000
+cpu_OSPKE = 0x0000e000
 
 #eventually, change this for your own codes
 #ADDEXP_SCALE_OFFSET= 0 
@@ -266,3 +267,18 @@ cpu_BMI   = 0x0000d000
 ADDRMETH_MASK = 0x00FF0000
 OPTYPE_MASK = 0xFF000000
 OPFLAGS_MASK = 0x0000FFFF
+
+# NOTE: some notes from the intel manual...
+# REX.W overrides 66, but alternate registers (via REX.B etc..) can have 66 to be 16 bit..
+# REX.R only modifies reg for GPR/SSE(SIMD)/ctrl/debug addressing modes.
+# REX.X only modifies the SIB index value
+# REX.B modifies modrm r/m field, or SIB base (if SIB present), or opcode reg.
+# We inherit all the regular intel prefixes...
+# VEX replaces REX, and mixing them is invalid
+PREFIX_REX   = 0x100000  # Shows that the rex prefix is present
+PREFIX_REX_B = 0x010000  # Bit 0 in REX prefix (0x41) means ModR/M r/m field, SIB base, or opcode reg
+PREFIX_REX_X = 0x020000  # Bit 1 in REX prefix (0x42) means SIB index extension
+PREFIX_REX_R = 0x040000  # Bit 2 in REX prefix (0x44) means ModR/M reg extention
+PREFIX_REX_W = 0x080000  # Bit 3 in REX prefix (0x48) means 64 bit operand
+PREFIX_REX_MASK = PREFIX_REX_B | PREFIX_REX_X | PREFIX_REX_W | PREFIX_REX_R
+PREFIX_REX_RXB  = PREFIX_REX_B | PREFIX_REX_X | PREFIX_REX_R
