@@ -2,28 +2,15 @@ import re
 import struct
 
 import envi
+import envi.exc as e_exc
 import envi.bits as e_bits
 
+from envi.const import *
 """
 A module containing memory utilities and the definition of the
 memory access API used by all vtoys trace/emulators/workspaces.
 """
 
-# Memory Map Permission Flags
-# TODO: move these into envi.const
-MM_NONE = 0x0
-MM_READ = 0x4
-MM_WRITE = 0x2
-MM_EXEC = 0x1
-MM_SHARED = 0x08
-
-MM_READ_WRITE = MM_READ | MM_WRITE
-MM_READ_EXEC = MM_READ | MM_EXEC
-MM_RWX = MM_READ | MM_WRITE | MM_EXEC
-
-pnames = ['No Access', 'Execute', 'Write', None, 'Read']
-
-
 def getPermName(perm):
     '''
     Return the human readable name for a *single* memory
@@ -419,7 +406,61 @@ class MemoryObject(IMemory):
         self._map_defs = []
         self._supervisor = False
 
-    #FIXME MemoryObject: def allocateMemory(self, size, perms=MM_RWX, suggestaddr=0):
+    def allocateMemory(self, size, perms=MM_RWX, suggestaddr=0x1000, name='', fill=b'\0'):
+        '''
+        Find a free block of memory (no maps exist) and allocate a new map
+        Uses findFreeMemoryBlock()
+        '''
+        baseva = self.findFreeMemoryBlock(size, suggestaddr)
+        self.addMemoryMap(baseva, perms, name, fill*size)
+        return baseva
+
+    def findFreeMemoryBlock(self, size, suggestaddr=0x1000, MIN_MEM_ADDR = 0x1000):
+        '''
+        Find a block of memory in the address-space of the correct size which 
+        doesn't overlap any existing maps.  Attempts to offer the map starting
+        at suggestaddr.  If not possible, scans the rest of the address-space
+        until it finds a suitable location or loops twice(ie. no gap large 
+        enough to accommodate a map of this size exists.
+
+        DOES NOT ALLOCATE.  see allocateMemory() if you want the map created
+        '''
+        baseva = None
+        looped = False
+
+        tmpva = suggestaddr
+        maxaddr = (1 << (8 * self.imem_psize)) - 1
+
+        while baseva is None:
+            # if we roll into illegal memory, start over at page 2.  skip 0.
+            if tmpva > maxaddr:
+                if looped:
+                    raise e_exc.NoValidFreeMemoryFound(size)
+
+                looped = True
+                tmpva = MIN_MEM_ADDR
+
+            # check potential map for any overlap
+            good = True
+            tmpendva = tmpva + size - 1
+            for mmva, mmsz, mmperm, mmname in self.getMemoryMaps():
+                mmendva = mmva + mmsz - 1
+                if tmpva <= mmva < tmpendva or \
+                        tmpva <= mmendva < tmpendva or \
+                        mmva <= tmpva < mmendva or \
+                        mmva <= tmpendva < mmendva:
+
+                    # we ran into a memory map.  adjust.
+                    good = False
+                    tmpva = mmendva
+                    tmpva += PAGE_NMASK
+                    tmpva &= PAGE_MASK
+                    break
+
+            if good:
+                baseva = tmpva
+
+        return baseva
 
     def addMemoryMap(self, va, perms, fname, bytez):
         '''
@@ -431,6 +472,16 @@ class MemoryObject(IMemory):
         self._map_defs.append(hlpr)
         return
 
+    def delMemoryMap(self, mapva):
+        '''
+        Delete a memory map from this object...
+        '''
+        for midx, (mva, mmaxva, mmap, mbytes) in enumerate(self._map_defs):
+            if mva == mapva:
+                return self._map_defs.pop(midx)
+
+        raise e_exc.MapNotFoundException(mapva)
+
     def getMemorySnap(self):
         '''
         Take a memory snapshot which may be restored later.
@@ -462,7 +513,9 @@ class MemoryObject(IMemory):
         return [mmap for mva, mmaxva, mmap, mbytes in self._map_defs]
 
     def readMemory(self, va, size):
-
+        '''
+        Read memory from maps stored in memory maps.
+        '''
         for mva, mmaxva, mmap, mbytes in self._map_defs:
             if mva <= va < mmaxva:
                 mva, msize, mperms, mfname = mmap
