@@ -4,9 +4,11 @@ country looking for pointers to interesting things.
 
 in a previous life, this analysis code lived inside VivWorkspace.analyze()
 """
-import sys
 import logging
-from vivisect.const import RTYPE_BASEPTR, LOC_POINTER, L_LTYPE
+
+import visgraph.exc as g_exc
+import visgraph.graphcore as g_core
+from vivisect.const import RTYPE_BASEPTR, LOC_POINTER
 
 logger = logging.getLogger(__name__)
 
@@ -14,7 +16,7 @@ def analyze(vw):
 
     logger.info('...analyzing pointers.')
 
-    done = []
+    done = {}
 
     # Let's analyze Relocations we know are pointers
     for rva, rtype in vw.reloc_by_va.items():
@@ -24,7 +26,7 @@ def analyze(vw):
         for xfr, xto, xtype, xinfo in vw.getXrefsFrom(rva):
             logger.debug('pointer(1): 0x%x -> 0x%x', xfr, xto)
             vw.analyzePointer(xto)
-            done.append((xfr, xto))
+            done[xfr] = xto
 
     # Now, we'll analyze the pointers placed by the file wrapper (ELF, PE, MACHO, etc...)
     for pva, tva, fname, pname in vw.getVaSetRows('PointersFromFile'):
@@ -34,7 +36,7 @@ def analyze(vw):
             else:
                 logger.debug('making pointer(2) 0x%x -> 0x%x (%r)', pva, tva, pname)
             vw.makePointer(pva, tva, follow=True)
-            done.append((pva, tva))
+            done[pva] = tva
 
     for lva, lsz, lt, li in vw.getLocations(LOC_POINTER):
         tva = vw.readMemoryPtr(lva)
@@ -44,14 +46,14 @@ def analyze(vw):
         if vw.getLocation(tva) is not None:
             # event if it's a pointer, we may not have made a name for it
             if vw.getName(lva) is None:
-                done.append((lva, tva))
+                done[lva] = tva
             continue
 
         logger.debug('following previously discovered pointer 0x%x -> 0x%x', lva, tva)
         try:
             logger.debug('pointer(3): 0x%x -> 0x%x', lva, tva)
             vw.followPointer(tva)
-            done.append((lva, tva))
+            done[lva] = tva
         except Exception as e:
             logger.error('followPointer() failed for 0x%.8x (pval: 0x%.8x) (err: %s)', lva, tva, e)
 
@@ -62,30 +64,45 @@ def analyze(vw):
         try:
             logger.debug('pointer(4): 0x%x -> 0x%x', addr, pval)
             vw.makePointer(addr, follow=True)
-            done.append((addr, pval))
+            done[addr] = pval
         except Exception as e:
             logger.error('makePointer() failed for 0x%.8x (pval: 0x%.8x) (err: %s)', addr, pval, e)
 
-    # Now let's see what these guys should be named (if anything)
-    for _, _ in done:
-        for ptr, tgt in done:
-            try:
-                pname = vw.getName(ptr)
-                if pname is not None:
-                    logger.debug('skipping renaming of ptr 0x%x (currently: %r)', ptr, pname)
-                    continue
+    # link the possible pointers up
+    pgraph = g_core.Graph()
+    while done:
+        ptr, tgt = done.popitem()
+        try:
+            pgraph.addNode(ptr)
+        except g_exc.DuplicateNode:
+            pass
+        try:
+            pgraph.addNode(tgt)
+        except g_exc.DuplicateNode:
+            pass
+        pgraph.addEdgeByNids(ptr, tgt)
 
-                loc = vw.getLocation(ptr)
-                if loc is not None and loc[L_LTYPE] != LOC_POINTER:
-                    logger.debug('skipping naming of 0x%x (no longer a pointer: %s)', ptr, vw.reprLocation(loc))
-                    continue
-                tgtname = vw.getName(tgt)
-                if tgtname is not None:
-                    name = vw._addNamePrefix(tgtname, tgt, 'ptr', '_') + '_%.8x' % ptr
-                    logger.debug('0x%x: adding name prefix: %r  (%r)', tgt, tgtname, name)
-                    vw.makeName(ptr, name)
-                else:
-                    logger.debug('0x%x: Skipping naming due to no target name' % tgt)
-            except Exception as e:
-                logger.error('naming failed (0x%x -> 0x%x), (err: %s)', ptr, tgt, e)
-                sys.excepthook(*sys.exc_info())
+    leafs = [node for node in pgraph.getNodes() if pgraph.isLeafNode(node)]
+    for leaf in leafs:
+        va, props = leaf
+        name = vw.getName(va)
+        # if there's not a name at the bottom, not work making ptr names
+        if not name:
+            continue
+        links = [(ptr, tgt) for edge, ptr, tgt, props in pgraph.getRefsToByNid(va)]
+        while links:
+            ptr, tgt = links.pop(0)
+            pname = vw.getName(ptr)
+            if pname:
+                continue
+            if not vw.isLocType(ptr, LOC_POINTER):
+                continue
+
+            tgtname = vw.getName(tgt)
+            if tgtname:
+                name = vw._addNamePrefix(tgtname, tgt, 'ptr', '_') + '_%.8x' % ptr
+                logger.debug('0x%x: adding name prefix: %r  (%r)', tgt, tgtname, name)
+                vw.makeName(ptr, name)
+
+            for edge, n1, n2, props in pgraph.getRefsToByNid(ptr):
+                links.append((n1, n2))
