@@ -129,7 +129,6 @@ class WorkspaceEmulator:
         self.path = self.newCodePathNode()
         self.curpath = self.path
         self.op = None
-        self.opcache = {}
         self.emumon = None
         self.psize = self.getPointerSize()
 
@@ -195,6 +194,8 @@ class WorkspaceEmulator:
             taints = [self.setVivTaint('funcstack', i * self.psize) for i in range(20)]
             taintbytes = b''.join([e_bits.buildbytes(taint, self.psize) for taint in taints])
 
+            self.stack_pointer -= len(taintbytes)
+            self.setStackCounter(self.stack_pointer)
             self.writeMemory(self.stack_pointer, taintbytes)
         else:
             existing_map_size = self.stack_map_top - self.stack_map_base
@@ -238,13 +239,7 @@ class WorkspaceEmulator:
         self.emumon = emumon
 
     def parseOpcode(self, va, arch=envi.ARCH_DEFAULT):
-        # We can make an opcode *faster* with the workspace because of
-        # getByteDef etc... use it.
-        op = self.opcache.get(va)
-        if op is None:
-            op = envi.Emulator.parseOpcode(self, va, arch=arch)
-            self.opcache[va] = op
-        return op
+        return self.vw.parseOpcode(va, arch=arch)
 
     def checkCall(self, starteip, endeip, op):
         """
@@ -335,7 +330,6 @@ class WorkspaceEmulator:
         # if we've already hunted this location down, know it's a table, and we've resolved it
         # step carefully so we don't conflate tables together
         xrefs = vw.getXrefsFrom(op.va, rtype=REF_CODE)
-        branches = []
         for bva, bflags in op.getBranches(emu=None):
             if bflags & envi.BR_TABLE and vw.getLocation(op.va) and len(xrefs):
                 for xrfrom, xrto, xrtype, xrflags in xrefs:
