@@ -1,18 +1,19 @@
 import logging
-from io import StringIO
+from io import BytesIO
 
 import PE
 import PE.carve as pe_carve
 
 import vstruct
 import vivisect
+import vivisect.exc as v_exc
 import vivisect.parsers as v_parsers
 # Steal symbol parsing from vtrace
 import vtrace  # needed only for setting the logging level
 import vtrace.platforms.win32 as vt_win32
 
 import envi.exc as e_exc
-import envi.memory as e_mem
+import envi.const as e_const
 import envi.symstore.symcache as e_symcache
 
 from vivisect.const import *
@@ -31,13 +32,21 @@ for mod in (PE, vtrace):
 # 0x183   DEC Alpha AXP
 
 
+def align(v, alignment):
+    remainder = v % alignment
+    if remainder == 0:
+        return v
+    else:
+        return v + (alignment - remainder)
+
+
 def parseFile(vw, filename, baseaddr=None):
     pe = PE.PE(open(filename, "rb"))
-    return loadPeIntoWorkspace(vw, pe, filename, baseaddr=baseaddr)
+    return loadPeIntoWorkspace(vw, pe, filename=filename, baseaddr=baseaddr)
 
 
 def parseBytes(vw, bytes, baseaddr=None):
-    fd = StringIO(bytes)
+    fd = BytesIO(bytes)
     fd.seek(0)
     pe = PE.PE(fd)
     return loadPeIntoWorkspace(vw, pe, baseaddr=baseaddr)
@@ -45,9 +54,8 @@ def parseBytes(vw, bytes, baseaddr=None):
 
 def parseMemory(vw, memobj, base):
     pe = PE.peFromMemoryObject(memobj, base)
-    mapbase, mapsize, perms, fname = memobj.getMemoryMap(base)
     # FIXME does the PE's load address get fixedup on rebase?
-    return loadPeIntoWorkspace(vw, pe, fname)
+    return loadPeIntoWorkspace(vw, pe, filename=None)
 
 
 def parseFd(vw, fd, filename=None, baseaddr=None):
@@ -66,7 +74,7 @@ arch_names = {
     PE.IMAGE_FILE_MACHINE_THUMB: 'thumb16',
 }
 
-defcalls = {
+archcalls = {
     'i386': 'cdecl',
     'amd64': 'msx64call',
     'arm': 'armcall',
@@ -87,11 +95,13 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
 
     arch = arch_names.get(mach)
     if arch is None:
-        raise Exception("Machine %.4x is not supported for PE!" % mach)
+        raise v_exc.InvalidArchitecture("PE", hex(mach))
 
     vw.setMeta('Architecture', arch)
     vw.setMeta('Format', 'pe')
     vw.parsedbin = pe
+    byts = pe.getFileBytes()
+    vw.setMeta('FileBytes', v_parsers.compressBytes(byts))
 
     platform = 'windows'
 
@@ -102,7 +112,7 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
 
     vw.setMeta('Platform', platform)
 
-    vw.setMeta('DefaultCall', defcalls.get(arch, 'unknown'))
+    vw.setMeta('DefaultCall', archcalls.get(arch, 'unknown'))
 
     # Set ourselves up for extended windows binary analysis
 
@@ -115,21 +125,19 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
     codesize = pe.IMAGE_NT_HEADERS.OptionalHeader.SizeOfCode
     codervamax = codebase+codesize
 
-    fvivname = filename
+    # grab the file bytes for hashing
+    pe.fd.seek(0)
+    fhash = v_parsers.md5Bytes(byts)
+    sha256 = v_parsers.sha256Bytes(byts)
 
+    fvivname = filename
     # This will help linkers with files that are re-named
     dllname = pe.getDllName()
-    if dllname is not None:
+    if dllname:
         fvivname = dllname
 
     if fvivname is None:
-        fvivname = "pe_%.8x" % baseaddr
-
-    # grab the file bytes for hashing
-    pe.fd.seek(0)
-    bytez = pe.fd.read()
-    fhash = v_parsers.md5Bytes(bytez)
-    sha256 = v_parsers.sha256Bytes(bytez)
+        fvivname = fhash
 
     # create the file and store md5 and sha256 hashes
     fname = vw.addFile(fvivname.lower(), baseaddr, fhash)
@@ -173,15 +181,22 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
     # Add the first page mapped in from the PE header.
     header = pe.readAtOffset(0, header_size)
 
+    if not header:
+        raise v_exc.CorruptPeFile("truncated PE header")
+
     secalign = pe.IMAGE_NT_HEADERS.OptionalHeader.SectionAlignment
+    filealign = pe.IMAGE_NT_HEADERS.OptionalHeader.FileAlignment
     subsys_majver = pe.IMAGE_NT_HEADERS.OptionalHeader.MajorSubsystemVersion
     subsys_minver = pe.IMAGE_NT_HEADERS.OptionalHeader.MinorSubsystemVersion
 
+    if secalign == 0:
+        raise v_exc.CorruptPeFile("section alignment is zero")
+
     secrem = len(header) % secalign
     if secrem != 0:
         header += b'\x00' * (secalign - secrem)
 
-    vw.addMemoryMap(baseaddr, e_mem.MM_READ, fname, header)
+    vw.addMemoryMap(baseaddr, e_const.MM_READ, fname, header)
     vw.addSegment(baseaddr, len(header), "PE_Header", fname)
 
     hstruct = vw.makeStructure(baseaddr, "pe.IMAGE_DOS_HEADER")
@@ -225,10 +240,13 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
 
     for idx, sec in enumerate(pe.sections):
         mapflags = 0
-
+        offset = sec.vsGetMeta("Offset", None)
+        if offset:
+            addr = baseaddr + offset
+            vw.makeStructure(addr, 'pe.IMAGE_SECTION_HEADER')
         chars = sec.Characteristics
         if chars & PE.IMAGE_SCN_MEM_READ:
-            mapflags |= e_mem.MM_READ
+            mapflags |= e_const.MM_READ
 
             isrsrc = (sec.VirtualAddress == ddir.VirtualAddress)
             if isrsrc and not loadrsrc:
@@ -242,16 +260,16 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
             # so we can't just blindly mark these as executable quite yet.
             if not nxcompat:
                 if not vw.config.viv.parsers.pe.nx and subsys_majver < 6 and not isrsrc:
-                    mapflags |= e_mem.MM_EXEC
+                    mapflags |= e_const.MM_EXEC
 
         if chars & PE.IMAGE_SCN_MEM_READ:
-            mapflags |= e_mem.MM_READ
+            mapflags |= e_const.MM_READ
         if chars & PE.IMAGE_SCN_MEM_WRITE:
-            mapflags |= e_mem.MM_WRITE
+            mapflags |= e_const.MM_WRITE
         if chars & PE.IMAGE_SCN_MEM_EXECUTE:
-            mapflags |= e_mem.MM_EXEC
+            mapflags |= e_const.MM_EXEC
         if chars & PE.IMAGE_SCN_CNT_CODE:
-            mapflags |= e_mem.MM_EXEC
+            mapflags |= e_const.MM_EXEC
 
         secrva = sec.VirtualAddress
         secvsize = sec.VirtualSize
@@ -263,16 +281,16 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
         # If the section is part of BaseOfCode->SizeOfCode
         # force execute perms...
         if secrva >= codebase and secrva < codervamax:
-            mapflags |= e_mem.MM_EXEC
+            mapflags |= e_const.MM_EXEC
 
         # If the entry point is in this section, force execute
         # permissions.
         if secrva <= entryrva and entryrva < secrvamax:
-            mapflags |= e_mem.MM_EXEC
+            mapflags |= e_const.MM_EXEC
 
         if not nxcompat:
-            if not vw.config.viv.parsers.pe.nx and subsys_majver < 6 and mapflags & e_mem.MM_READ:
-                mapflags |= e_mem.MM_EXEC
+            if not vw.config.viv.parsers.pe.nx and subsys_majver < 6 and mapflags & e_const.MM_READ:
+                mapflags |= e_const.MM_EXEC
 
         if sec.VirtualSize == 0 or sec.SizeOfRawData == 0:
             if idx+1 >= len(pe.sections):
@@ -309,12 +327,19 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
             # According to http://code.google.com/p/corkami/wiki/PE#section_table if SizeOfRawData is larger than VirtualSize, VS is used..
             readsize = sec.SizeOfRawData if sec.SizeOfRawData < sec.VirtualSize else sec.VirtualSize
 
+            # align the read to the FileAlignment
+            readsize = align(readsize, filealign)
+
             secoff = pe.rvaToOffset(secrva)
             secbytes = pe.readAtOffset(secoff, readsize, shortok=True)
             slen = len(secbytes)
             if slen != readsize:
                 logger.warning("Section at offset 0x%x should have 0x%x bytes, but we only got 0x%x bytes", secoff, readsize, slen)
-            secbytes += b'\x00' * plen
+
+            if slen != align(sec.VirtualSize, secalign):
+                # pad the section up to next the SectionAlignment
+                secbytes += b'\x00' * (align(sec.VirtualSize, secalign) - slen)
+
             slen = len(secbytes)
             vw.addMemoryMap(secbase, mapflags, fname, secbytes)
             vw.addSegment(secbase, slen, secname, fname)
@@ -364,11 +389,11 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
             vw.addRelocation(rva + baseaddr, vtype, mapoffset)
 
     for rva, lname, iname in pe.getImports():
-        if vw.probeMemory(rva + baseaddr, 4, e_mem.MM_READ):
+        if vw.probeMemory(rva + baseaddr, 4, e_const.MM_READ):
             vw.makeImport(rva + baseaddr, lname, iname)
 
     for rva, lname, iname in pe.getDelayImports():
-        if vw.probeMemory(rva + baseaddr, 4, e_mem.MM_READ):
+        if vw.probeMemory(rva + baseaddr, 4, e_const.MM_READ):
             vw.makeImport(rva + baseaddr, lname, iname)
             if lname != '*':
                 lname = vw.normFileName(lname)
@@ -379,17 +404,17 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
     vw.addNoReturnApi("kernel32.ExitProcess")
     vw.addNoReturnApi("kernel32.ExitThread")
     vw.addNoReturnApi("kernel32.FatalExit")
-    vw.addNoReturnApiRegex("^msvcr.*\._CxxThrowException$")
-    vw.addNoReturnApiRegex("^msvcr.*\.abort$")
-    vw.addNoReturnApiRegex("^msvcr.*\.exit$")
-    vw.addNoReturnApiRegex("^msvcr.*\._exit$")
-    vw.addNoReturnApiRegex("^msvcr.*\.quick_exit$")
+    vw.addNoReturnApiRegex(r"^msvcr.*\._CxxThrowException$")
+    vw.addNoReturnApiRegex(r"^msvcr.*\.abort$")
+    vw.addNoReturnApiRegex(r"^msvcr.*\.exit$")
+    vw.addNoReturnApiRegex(r"^msvcr.*\._exit$")
+    vw.addNoReturnApiRegex(r"^msvcr.*\.quick_exit$")
+    vw.addNoReturnApiRegex(r"^api_ms_win_crt_runtime_.*\._invalid_parameter_noinfo_noreturn$")
+    vw.addNoReturnApiRegex(r"^api_ms_win_crt_runtime_.*\.exit$")
+    vw.addNoReturnApiRegex(r"^api_ms_win_crt_runtime_.*\._exit$")
     # https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/invalid-parameter-functions?view=vs-2019
     # TODO: Again, there's a couple in there that have conditional termination that we should check for
     #vw.addNoReturnApiRegex("vcruntime140.__std_terminate")
-    vw.addNoReturnApiRegex("^api_ms_win_crt_runtime_.*\._invalid_parameter_noinfo_noreturn$")
-    vw.addNoReturnApiRegex("^api_ms_win_crt_runtime_.*\.exit$")
-    vw.addNoReturnApiRegex("^api_ms_win_crt_runtime_.*\._exit$")
     # TODO: we should add abort and terminate on the conditions that there are no signal handlers
     # registered
     # https://docs.microsoft.com/en-us/cpp/c-runtime-library/reference/cexit-c-exit?view=vs-2019
@@ -409,7 +434,7 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
         try:
             vw.setVaSetRow('pe:ordinals', (eva, ord))
             vw.addExport(eva, EXP_UNTYPED, name, fname)
-            if vw.probeMemory(eva, 1, e_mem.MM_EXEC):
+            if vw.probeMemory(eva, 1, e_const.MM_EXEC):
                 vw.addEntryPoint(eva)
         except Exception as e:
             vw.vprint('addExport Failed: %s.%s (0x%.8x): %s' % (fname, name, eva, e))
@@ -447,7 +472,7 @@ def loadPeIntoWorkspace(vw, pe, filename=None, baseaddr=None):
                     vw.vprint("SEHandlerTable parse error")
 
     # Last but not least, see if we have symbol support and use it if we do
-    if vt_win32.dbghelp:
+    if vt_win32.dbghelp and filename:
 
         s = vt_win32.Win32SymbolParser(-1, filename, baseaddr)
 
