@@ -27,11 +27,11 @@ class MalwareTests(unittest.TestCase):
     def test_memorymaps_basicperms(self):
         vw = self.mmap_vw
         maps = set([
-            (268435456, 4096, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
-            (268439552, 420578, 5, '0e25aa791c9119108af073bc9e9d0fa2'),
-            (268861440, 56256, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
-            (268918784, 79100, 6, '0e25aa791c9119108af073bc9e9d0fa2'),
-            (269004800, 17638, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
+            (268435456, 0x1000, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
+            (268439552, 0x67000, 5, '0e25aa791c9119108af073bc9e9d0fa2'),
+            (268861440, 0xe000, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
+            (268918784, 0x14000, 6, '0e25aa791c9119108af073bc9e9d0fa2'),
+            (269004800, 0x5000, 4, '0e25aa791c9119108af073bc9e9d0fa2'),
         ])
         self.assertEqual(maps, set(vw.getMemoryMaps()))
 
@@ -40,7 +40,7 @@ class MalwareTests(unittest.TestCase):
         vw = self.mmap_vw
         for lva, lsize, ltype, linfo in vw.getLocations():
             memmap = vw.getMemoryMap(lva)
-            if memmap[0] != 268439552:
+            if memmap[0] != 0x10001000:
                 self.assertNotEqual(ltype, 5)
 
     def test_carve_pe(self):
