@@ -492,10 +492,10 @@ class PETests(unittest.TestCase):
         ans = {
             # name -> (Base, Size, Flags)
             'PE_Header': (0x400000, 0x1000, e_memory.MM_READ),
-            '.text': (0x401000, 0x71635, e_memory.MM_READ | e_memory.MM_EXEC),
-            '.data': (0x4b6000, 0x41c8, e_memory.MM_READ | e_memory.MM_WRITE),
-            '.rdata': (0x473000, 0x42ca6, e_memory.MM_READ),
-            '.reloc': (0x4bf000, 0x661a, e_memory.MM_READ),
+            '.text': (0x401000, 0x72000, e_memory.MM_READ | e_memory.MM_EXEC),
+            '.data': (0x4b6000, 0x5000, e_memory.MM_READ | e_memory.MM_WRITE),
+            '.rdata': (0x473000, 0x43000, e_memory.MM_READ),
+            '.reloc': (0x4bf000, 0x7000, e_memory.MM_READ),
         }
         for sva, ssize, sname, sfname in vw.getSegments():
             self.assertEqual(ans[sname][0], sva)
@@ -507,3 +507,20 @@ class PETests(unittest.TestCase):
             self.assertEqual(msize, ssize)
             self.assertEqual(flags, ans[sname][2])
             self.assertEqual(mfname, sfname)
+
+    def test_saved_structures(self):
+        ans = [
+            0x4001f0,
+            0x400218,
+            0x400240,
+            0x400268,
+            0x400290
+        ]
+        vw = self.vw_psexec
+        for va in ans:
+            loc = vw.getLocation(va)
+            self.assertIsNotNone(loc)
+            self.assertEqual(loc[0], va)
+            self.assertEqual(loc[1], 40)
+            self.assertEqual(loc[2], viv_con.LOC_STRUCT)
+            self.assertEqual(loc[3], 'pe.IMAGE_SECTION_HEADER')
