@@ -1,5 +1,15 @@
-# Version: 5.1
+# -*- coding: utf8 -*-
+# Version: 5.2
 # Architecture: i386
+# CompanyName: Microsoft Corporation
+# FileDescription: Multi-User Win32 Driver
+# FileVersion: 5.2.3790.5019 (srv03_sp2_gdr.120613-0334)
+# InternalName: win32k.sys
+# LegalCopyright:  Microsoft Corporation. All rights reserved.
+# OriginalFilename: win32k.sys
+# ProductName: Microsoft Windows Operating System
+# ProductVersion: 5.2.3790.5019
+# Translation: 78644233
 import vstruct
 from vstruct.primitives import *
 
@@ -35,6 +45,28 @@ DEVICE_POWER_STATE.PowerDeviceD3 = 4
 DEVICE_POWER_STATE.PowerDeviceMaximum = 5
 
 
+KSPIN_LOCK_QUEUE_NUMBER = v_enum()
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueDispatcherLock = 0
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueUnusedSpare1 = 1
+KSPIN_LOCK_QUEUE_NUMBER.LockQueuePfnLock = 2
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueSystemSpaceLock = 3
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueVacbLock = 4
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueMasterLock = 5
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueNonPagedPoolLock = 6
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueIoCancelLock = 7
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueWorkQueueLock = 8
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueIoVpbLock = 9
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueIoDatabaseLock = 10
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueIoCompletionLock = 11
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueNtfsStructLock = 12
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueAfdWorkQueueLock = 13
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueBcbLock = 14
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueMmNonPagedPoolLock = 15
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueUnusedSpare16 = 16
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueTimerTableLock = 17
+KSPIN_LOCK_QUEUE_NUMBER.LockQueueMaximumLock = 33
+
+
 FSINFOCLASS = v_enum()
 FSINFOCLASS.FileFsVolumeInformation = 1
 FSINFOCLASS.FileFsLabelInformation = 2
@@ -167,7 +199,8 @@ FILE_INFORMATION_CLASS.FileIdBothDirectoryInformation = 37
 FILE_INFORMATION_CLASS.FileIdFullDirectoryInformation = 38
 FILE_INFORMATION_CLASS.FileValidDataLengthInformation = 39
 FILE_INFORMATION_CLASS.FileShortNameInformation = 40
-FILE_INFORMATION_CLASS.FileMaximumInformation = 41
+FILE_INFORMATION_CLASS.FileIoCompletionNotificationInformation = 41
+FILE_INFORMATION_CLASS.FileMaximumInformation = 42
 
 
 GDILoObjType = v_enum()
@@ -268,6 +301,18 @@ ReplacesCorHdrNumericDefines.MAX_CLASS_NAME = 1024
 ReplacesCorHdrNumericDefines.MAX_PACKAGE_NAME = 1024
 
 
+EX_POOL_PRIORITY = v_enum()
+EX_POOL_PRIORITY.LowPoolPriority = 0
+EX_POOL_PRIORITY.LowPoolPrioritySpecialPoolOverrun = 8
+EX_POOL_PRIORITY.LowPoolPrioritySpecialPoolUnderrun = 9
+EX_POOL_PRIORITY.NormalPoolPriority = 16
+EX_POOL_PRIORITY.NormalPoolPrioritySpecialPoolOverrun = 24
+EX_POOL_PRIORITY.NormalPoolPrioritySpecialPoolUnderrun = 25
+EX_POOL_PRIORITY.HighPoolPriority = 32
+EX_POOL_PRIORITY.HighPoolPrioritySpecialPoolOverrun = 40
+EX_POOL_PRIORITY.HighPoolPrioritySpecialPoolUnderrun = 41
+
+
 SYSTEM_POWER_STATE = v_enum()
 SYSTEM_POWER_STATE.PowerSystemUnspecified = 0
 SYSTEM_POWER_STATE.PowerSystemWorking = 1
@@ -305,13 +350,14 @@ class OBJECT_ATTRIBUTES(vstruct.VStruct):
         self.SecurityQualityOfService = v_ptr32()
 
 
-class SECURITY_SUBJECT_CONTEXT(vstruct.VStruct):
+class tagWin32AllocStats(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.ClientToken = v_ptr32()
-        self.ImpersonationLevel = v_uint32()
-        self.PrimaryToken = v_ptr32()
-        self.ProcessAuditId = v_ptr32()
+        self.dwMaxMem = v_uint32()
+        self.dwCrtMem = v_uint32()
+        self.dwMaxAlloc = v_uint32()
+        self.dwCrtAlloc = v_uint32()
+        self.pHead = v_ptr32()
 
 
 class CM_FULL_RESOURCE_DESCRIPTOR(vstruct.VStruct):
@@ -355,10 +401,21 @@ class FAST_IO_DISPATCH(vstruct.VStruct):
         self.ReleaseForCcFlush = v_ptr32()
 
 
-class ACCESS_STATE(vstruct.VStruct):
+class SECTION_OBJECT_POINTERS(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.InitialPrivilegeSet = INITIAL_PRIVILEGE_SET()
+        self.DataSectionObject = v_ptr32()
+        self.SharedCacheMap = v_ptr32()
+        self.ImageSectionObject = v_ptr32()
+
+
+class tagWin32PoolHead(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.size = v_uint32()
+        self.pPrev = v_ptr32()
+        self.pNext = v_ptr32()
+        self.pTrace = v_ptr32()
 
 
 class LARGE_INTEGER(vstruct.VStruct):
@@ -401,7 +458,12 @@ class EX_PUSH_LOCK_CACHE_AWARE(vstruct.VStruct):
 class EX_PUSH_LOCK(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Waiting = v_uint32()
+        self.Locked = v_uint32()
+
+
+class DEVICE_MAP(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
 
 
 class IMAGE_NT_HEADERS(vstruct.VStruct):
@@ -465,22 +527,16 @@ class FAST_MUTEX(vstruct.VStruct):
         self.Count = v_uint32()
         self.Owner = v_ptr32()
         self.Contention = v_uint32()
-        self.Event = KEVENT()
+        self.Gate = KEVENT()
         self.OldIrql = v_uint32()
 
 
-class VPB(vstruct.VStruct):
+class CM_PARTIAL_RESOURCE_DESCRIPTOR(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Type = v_uint16()
-        self.Size = v_uint16()
-        self.Flags = v_uint16()
-        self.VolumeLabelLength = v_uint16()
-        self.DeviceObject = v_ptr32()
-        self.RealDevice = v_ptr32()
-        self.SerialNumber = v_uint32()
-        self.ReferenceCount = v_uint32()
-        self.VolumeLabel = vstruct.VArray([ v_uint16() for i in range(32) ])
+        self.DataSize = v_uint32()
+        self.Reserved1 = v_uint32()
+        self.Reserved2 = v_uint32()
 
 
 class IO_RESOURCE_DESCRIPTOR(vstruct.VStruct):
@@ -489,14 +545,13 @@ class IO_RESOURCE_DESCRIPTOR(vstruct.VStruct):
         self.Port = IO_RESOURCE_DESCRIPTOR()
 
 
-class tagWin32AllocStats(vstruct.VStruct):
+class SECURITY_SUBJECT_CONTEXT(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.dwMaxMem = v_uint32()
-        self.dwCrtMem = v_uint32()
-        self.dwMaxAlloc = v_uint32()
-        self.dwCrtAlloc = v_uint32()
-        self.pHead = v_ptr32()
+        self.ClientToken = v_ptr32()
+        self.ImpersonationLevel = v_uint32()
+        self.PrimaryToken = v_ptr32()
+        self.ProcessAuditId = v_ptr32()
 
 
 class KDEVICE_QUEUE(vstruct.VStruct):
@@ -703,6 +758,18 @@ class OBJECT_HANDLE_INFORMATION(vstruct.VStruct):
         self.GrantedAccess = v_uint32()
 
 
+class tagVSTATE(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.fl = v_uint32()
+        self.bSystemStable = v_uint32()
+        self.ulRandomSeed = v_uint32()
+        self.ulFailureMask = v_uint32()
+        self.ulDebugLevel = v_uint32()
+        self.hsemPoolTracker = v_ptr32()
+        self.lePoolTrackerHead = LIST_ENTRY()
+
+
 class IRP(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -718,13 +785,6 @@ class INITIAL_PRIVILEGE_SET(vstruct.VStruct):
         self.Privilege = vstruct.VArray([ LUID_AND_ATTRIBUTES() for i in range(3) ])
 
 
-class ULARGE_INTEGER(vstruct.VStruct):
-    def __init__(self):
-        vstruct.VStruct.__init__(self)
-        self.LowPart = v_uint32()
-        self.HighPart = v_uint32()
-
-
 class GENERAL_LOOKASIDE(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -746,10 +806,17 @@ class GENERAL_LOOKASIDE(vstruct.VStruct):
         self.Future = vstruct.VArray([ v_uint32() for i in range(2) ])
 
 
+class ULARGE_INTEGER(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.LowPart = v_uint32()
+        self.HighPart = v_uint32()
+
+
 class EX_PUSH_LOCK(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Waiting = v_uint32()
+        self.Locked = v_uint32()
 
 
 class CM_PARTIAL_RESOURCE_DESCRIPTOR(vstruct.VStruct):
@@ -758,12 +825,19 @@ class CM_PARTIAL_RESOURCE_DESCRIPTOR(vstruct.VStruct):
         self.Generic = CM_PARTIAL_RESOURCE_DESCRIPTOR()
 
 
-class SECTION_OBJECT_POINTERS(vstruct.VStruct):
+class OBJECT_DIRECTORY(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.DataSectionObject = v_ptr32()
-        self.SharedCacheMap = v_ptr32()
-        self.ImageSectionObject = v_ptr32()
+        self.HashBuckets = vstruct.VArray([ v_ptr32() for i in range(37) ])
+        self.Lock = EX_PUSH_LOCK()
+        self.DeviceMap = v_ptr32()
+        self.SessionId = v_uint32()
+
+
+class DISPATCHER_HEADER(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.Inserted = v_uint8()
 
 
 class DEVOBJ_EXTENSION(vstruct.VStruct):
@@ -774,22 +848,10 @@ class DEVOBJ_EXTENSION(vstruct.VStruct):
         self.DeviceObject = v_ptr32()
 
 
-class tagVSTATE(vstruct.VStruct):
+class IRP(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.fl = v_uint32()
-        self.bSystemStable = v_uint32()
-        self.ulRandomSeed = v_uint32()
-        self.ulFailureMask = v_uint32()
-        self.ulDebugLevel = v_uint32()
-        self.hsemPoolTracker = v_ptr32()
-        self.lePoolTrackerHead = LIST_ENTRY()
-
-
-class EX_RUNDOWN_REF(vstruct.VStruct):
-    def __init__(self):
-        vstruct.VStruct.__init__(self)
-        self.Count = v_uint32()
+        self.CurrentStackLocation = v_ptr32()
 
 
 class POWER_STATE(vstruct.VStruct):
@@ -834,6 +896,11 @@ class ACCESS_STATE(vstruct.VStruct):
         self.ObjectTypeName = UNICODE_STRING()
 
 
+class OBJECT_DIRECTORY_ENTRY(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+
+
 class FILE_STANDARD_INFORMATION(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -848,9 +915,11 @@ class FILE_STANDARD_INFORMATION(vstruct.VStruct):
 class KAPC(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Type = v_uint16()
-        self.Size = v_uint16()
-        self.Spare0 = v_uint32()
+        self.Type = v_uint8()
+        self.SpareByte0 = v_uint8()
+        self.Size = v_uint8()
+        self.SpareByte1 = v_uint8()
+        self.SpareLong0 = v_uint32()
         self.Thread = v_ptr32()
         self.ApcListEntry = LIST_ENTRY()
         self.KernelRoutine = v_ptr32()
@@ -877,10 +946,27 @@ class WAIT_CONTEXT_BLOCK(vstruct.VStruct):
         self.BufferChainingDpc = v_ptr32()
 
 
+class OBJECT_HEADER_NAME_INFO(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.Directory = v_ptr32()
+        self.Name = UNICODE_STRING()
+        self.QueryReferences = v_uint32()
+
+
+class DISPATCHER_HEADER(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.Type = v_uint8()
+        self.Absolute = v_uint8()
+        self.Size = v_uint8()
+        self.Inserted = v_uint8()
+
+
 class EX_PUSH_LOCK(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Waiting = v_uint32()
+        self.Locked = v_uint32()
 
 
 class MDL(vstruct.VStruct):
@@ -896,12 +982,6 @@ class MDL(vstruct.VStruct):
         self.ByteOffset = v_uint32()
 
 
-class EX_RUNDOWN_REF(vstruct.VStruct):
-    def __init__(self):
-        vstruct.VStruct.__init__(self)
-        self.Count = v_uint32()
-
-
 class W32THREAD(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -912,23 +992,23 @@ class W32THREAD(vstruct.VStruct):
         self.pgdiBrushAttr = v_ptr32()
         self.pUMPDObjs = v_ptr32()
         self.pUMPDHeap = v_ptr32()
-        self.dwEngAcquireCount = v_uint32()
-        self.pSemTable = v_ptr32()
         self.pUMPDObj = v_ptr32()
+        self.GdiTmpAllocList = LIST_ENTRY()
 
 
 class KDPC(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Type = v_uint16()
-        self.Number = v_uint8()
+        self.Type = v_uint8()
         self.Importance = v_uint8()
+        self.Number = v_uint8()
+        self.Expedite = v_uint8()
         self.DpcListEntry = LIST_ENTRY()
         self.DeferredRoutine = v_ptr32()
         self.DeferredContext = v_ptr32()
         self.SystemArgument1 = v_ptr32()
         self.SystemArgument2 = v_ptr32()
-        self.Lock = v_ptr32()
+        self.DpcData = v_ptr32()
 
 
 class OWNER_ENTRY(vstruct.VStruct):
@@ -993,10 +1073,10 @@ class DISPATCHER_HEADER(vstruct.VStruct):
         self.WaitListHead = LIST_ENTRY()
 
 
-class IRP(vstruct.VStruct):
+class QUAD(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.CurrentStackLocation = v_ptr32()
+        self.UseThisFieldToCopy = v_uint64()
 
 
 class HOBJ(vstruct.VStruct):
@@ -1010,6 +1090,12 @@ class IO_TIMER(vstruct.VStruct):
         vstruct.VStruct.__init__(self)
 
 
+class OBJECT_HEADER(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.ObjectCreateInfo = v_ptr32()
+
+
 class OBJECT_TYPE_INITIALIZER(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -1135,6 +1221,11 @@ class BASEOBJECT(vstruct.VStruct):
         self.Tid = v_ptr32()
 
 
+class OBJECT_CREATE_INFORMATION(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+
+
 class HSEMAPHORE(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -1148,6 +1239,27 @@ class LUID(vstruct.VStruct):
         self.HighPart = v_uint32()
 
 
+class IO_STACK_LOCATION(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.Create = IO_STACK_LOCATION()
+
+
+class OBJECT_HEADER(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.PointerCount = v_uint32()
+        self.HandleCount = v_uint32()
+        self.Type = v_ptr32()
+        self.NameInfoOffset = v_uint8()
+        self.HandleInfoOffset = v_uint8()
+        self.QuotaInfoOffset = v_uint8()
+        self.Flags = v_uint8()
+        self.ObjectCreateInfo = v_ptr32()
+        self.SecurityDescriptor = v_ptr32()
+        self.Body = QUAD()
+
+
 class LARGE_INTEGER(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -1163,6 +1275,12 @@ class tagPOOLRECORD(vstruct.VStruct):
         self.trace = vstruct.VArray([ v_ptr32() for i in range(6) ])
 
 
+class QUAD(vstruct.VStruct):
+    def __init__(self):
+        vstruct.VStruct.__init__(self)
+        self.UseThisFieldToCopy = v_uint64()
+
+
 class GUID(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -1200,13 +1318,10 @@ class OBJECTOWNER(vstruct.VStruct):
         self.Share = OBJECTOWNER_S()
 
 
-class tagWin32PoolHead(vstruct.VStruct):
+class ACCESS_STATE(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.size = v_uint32()
-        self.pPrev = v_ptr32()
-        self.pNext = v_ptr32()
-        self.pTrace = v_ptr32()
+        self.InitialPrivilegeSet = INITIAL_PRIVILEGE_SET()
 
 
 class TL(vstruct.VStruct):
@@ -1226,12 +1341,18 @@ class IO_STACK_LOCATION(vstruct.VStruct):
         self.Argument4 = v_ptr32()
 
 
-class CM_PARTIAL_RESOURCE_DESCRIPTOR(vstruct.VStruct):
+class VPB(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.DataSize = v_uint32()
-        self.Reserved1 = v_uint32()
-        self.Reserved2 = v_uint32()
+        self.Type = v_uint16()
+        self.Size = v_uint16()
+        self.Flags = v_uint16()
+        self.VolumeLabelLength = v_uint16()
+        self.DeviceObject = v_ptr32()
+        self.RealDevice = v_ptr32()
+        self.SerialNumber = v_uint32()
+        self.ReferenceCount = v_uint32()
+        self.VolumeLabel = vstruct.VArray([ v_uint16() for i in range(32) ])
 
 
 class ERESOURCE(vstruct.VStruct):
@@ -1262,14 +1383,6 @@ class IO_RESOURCE_DESCRIPTOR(vstruct.VStruct):
         self.Reserved2 = v_uint32()
 
 
-class STRING(vstruct.VStruct):
-    def __init__(self):
-        vstruct.VStruct.__init__(self)
-        self.Length = v_uint16()
-        self.MaximumLength = v_uint16()
-        self.Buffer = v_ptr32()
-
-
 class GENERIC_MAPPING(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
@@ -1412,10 +1525,13 @@ class DRIVER_EXTENSION(vstruct.VStruct):
         self.ServiceKeyName = UNICODE_STRING()
 
 
-class IO_STACK_LOCATION(vstruct.VStruct):
+class DISPATCHER_HEADER(vstruct.VStruct):
     def __init__(self):
         vstruct.VStruct.__init__(self)
-        self.Create = IO_STACK_LOCATION()
+        self.Type = v_uint8()
+        self.Absolute = v_uint8()
+        self.Size = v_uint8()
+        self.Inserted = v_uint8()
 
 
 class KDEVICE_QUEUE_ENTRY(vstruct.VStruct):
